@@ -20,10 +20,11 @@ from EvoMSA.utils import download
 from collections import OrderedDict, defaultdict
 from microtc.utils import Counter
 from os.path import isfile, join, dirname
-from microtc.textmodel import TextModel
 from microtc.params import OPTION_DELETE, OPTION_NONE
 from microtc.utils import tweet_iterator
-from .place import BoundingBox, location
+from text_models.place import BoundingBox, location
+from text_models.utils import get_text
+import random
 
 
 try:
@@ -382,3 +383,189 @@ class GeoFrequency(object):
             data[key].clean()
             if len(data[key].counter) == 0 or data[key].num_documents <= min_value:
                 del data[key]
+
+class MaskedLMDataset(object):
+    """Create a masked language model
+    >>> from text_models.dataset import Dataset, MaskedLMDataset
+    >>> from EvoMSA.utils import download
+    >>> from microtc.utils import load_model
+    >>> from text_models.tests.test_dataset import TWEETS
+    >>> emo = load_model(download("emo_En.tm"))
+    >>> ds = Dataset()
+    >>> ds.add({k: True for k in emo._labels})
+    >>> mk = MaskedLMDataset(ds)
+    >>> mk.process(TWEETS)
+    """
+    def __init__(self, dataset: Dataset,
+                 num_elements: int=64000,
+                 lang="es",
+                 reader: Callable[[str], Iterable[dict]]=tweet_iterator) -> None:
+        self._dataset = dataset
+        self._reader = reader
+        self._store = defaultdict(list)
+        self._num_elements = num_elements
+        self._lang = lang
+
+    @property
+    def dataset(self):
+        return self._store
+
+    def missing(self):
+        """Number of klass that has not been reached the 
+        maximum number of elements
+        >>> from text_models.dataset import MaskedLMDataset
+        >>> ms = MaskedLMDataset(None, num_elements=64000)
+        >>> ms.add(['☹'], "aa ☹ b")
+        >>> ms.add(['☹', '☺'], "aa ☹ ☺ b")
+        >>> _ = [ms.add(['☹'], "aa ☹ b") for _ in range(64010)]
+        >>> ms.missing()
+        1
+        """
+        n = self._num_elements
+        _ = [1 for v in self.dataset.values() if len(v) < n]
+        return sum(_)
+
+    def add(self, klasses, text):
+        """Add text into the dataset
+        >>> from text_models.dataset import MaskedLMDataset
+        >>> ms = MaskedLMDataset(None, num_elements=64000)
+        >>> ms.add(['☹'], "aa ☹ b")
+        >>> len(ms.dataset['☹'])
+        1
+        >>> ms.add(['☹', '☺'], "aa ☹ ☺ b")
+        >>> len(ms.dataset['☺'])
+        1
+        >>> len(ms.dataset['☹'])
+        1
+        >>> _ = [ms.add(['☹'], "aa ☹ b") for _ in range(64010)]
+        >>> len(ms.dataset['☹'])
+        64000
+        """
+        if len(klasses) == 0:
+            return
+        dataset = self.dataset
+        num_elements = self._num_elements
+        lst = [(k, len(dataset[k])) for k in klasses]
+        lst = [(k, v) for k, v in lst if v < num_elements]
+        lst.sort(key=lambda x: x[1])
+        if len(lst):
+            dataset[lst[0][0]].append(text)
+
+    def filter(self, text):
+        """Filter RT
+        >>> from text_models.dataset import MaskedLMDataset
+        >>> ms = MaskedLMDataset(None)
+        >>> ms.filter('RT a')
+        True
+        >>> ms.filter('a')
+        False
+        """
+        if text[:2] == 'RT':
+            return True
+        return False
+
+    def lang(self, tw: dict):
+        """Test the language of the tweet
+        >>> from text_models.dataset import MaskedLMDataset
+        >>> ms = MaskedLMDataset(None, lang='ar')
+        >>> ms.lang(dict(lang="ar"))
+        True
+        >>> ms.lang(dict(lang="en"))
+        False
+        """
+        return tw.get('lang', "") == self._lang
+
+    def process(self, fname: str):
+        dataset = self._dataset
+        filter = self.filter
+        lang = self.lang
+        for tw in self._reader(fname):
+            if not lang(tw):
+                continue
+            text = get_text(tw)
+            if filter(text):
+                continue
+            klasses = dataset.klass(text)
+            self.add(klasses, text)
+
+
+class MaskedLM(object):
+    """Create a masked language model
+    >>> from text_models.dataset import Dataset, MaskedLMDataset, MaskedLM
+    >>> from EvoMSA.utils import download
+    >>> from microtc.utils import load_model
+    >>> from text_models.tests.test_dataset import TWEETS
+    >>> emo = load_model(download("emo_En.tm"))
+    >>> ds = Dataset()
+    >>> ds.add({k: True for k in emo._labels})
+    >>> mkDS = MaskedLMDataset(ds)
+    >>> mkDS.process(TWEETS)
+    >>> mk = MaskedLM(mkDS)
+    >>> tm = mk.textModel()
+    >>> coef, intercept, labels = mk.run(tm)
+    """
+    def __init__(self, masked: MaskedLMDataset) -> None:
+        self._masked = masked
+
+    def run(self, tm):
+        from sklearn.svm import LinearSVC
+        from EvoMSA.utils import linearSVC_array
+        dataset = self._masked.dataset
+        klasses = sorted(dataset.keys())
+        cnt = len(klasses) - 1
+        transform = self.transform
+        MODELS = []
+        for klass in tqdm(klasses):
+            X = transform(klass, dataset[klass])
+            y = [1] * len(X)
+            nneg = max(len(X) // cnt, 1)
+            for k2 in klasses:
+                if k2 == klass:
+                    continue
+                ele = dataset[klass]
+                random.shuffle(ele)
+                ele = transform(k2, ele[:nneg])
+                y += [-1] * len(ele)
+                X += ele
+            m = LinearSVC().fit(tm.transform(X), y)
+            MODELS.append(m)
+        coef, intercept = linearSVC_array(MODELS)
+        return coef, intercept, klasses
+
+    def transform(self, klass, texts) -> None:
+        """Transform
+        >>> from text_models.dataset import Dataset, MaskedLMDataset, MaskedLM
+        >>> from EvoMSA.utils import download
+        >>> from microtc.utils import load_model
+        >>> from text_models.tests.test_dataset import TWEETS
+        >>> emo = load_model(download("emo_En.tm"))
+        >>> ds = Dataset()
+        >>> ds.add({k: True for k in emo._labels})
+        >>> mk = MaskedLMDataset(ds)
+        >>> mk.process(TWEETS)
+        >>> mask = MaskedLM(mk)
+        >>> mask.transform('🤘', ['Tengo sueñoooooooo🤘', '🤘'])
+        [['tengo suenoooooooo']]
+        """
+        output = []
+        process = self._masked._dataset.process
+        for text in texts:
+            _ = [" ".join(filter(lambda x: len(x), x.split('~'))) for x in process(text)]
+            _ = list(filter(lambda x: len(x), _))
+            if len(_):
+                output.append(_)
+        return output
+
+    def textModel(self, n=10000):
+        tm = TokenCount.textModel([-2, -1, 2, 3, 4])
+        tm.token_min_filter = 0.001
+        tm.token_max_filter = 0.999
+        D = []
+        transform = self.transform
+        items = self._masked.dataset.items
+        for k, v in items():
+            random.shuffle(v)
+            D += transform(k, v[:n])
+        tm.fit(D)
+        return tm
+        
