@@ -292,9 +292,6 @@ class Vocabulary(object):
         for k in K:
             del self.voc[k]  
 
-    def remove_qgrams(self):
-        pass
-
     def histogram(self, min_elements: int=30, words: bool=False):
         group = defaultdict(list)
         [group[v].append(k) for k, v in self.voc.items() if words or k.count("~")]
@@ -313,6 +310,36 @@ class Vocabulary(object):
             hist[k] = lst
         return hist
 
+    @staticmethod
+    def available_dates(dates=List, n=int, countries=List, lang=str):
+        """Retrieve the first n dates available for all the countries
+
+        :param dates: List of dates
+        :param n: Number of days
+        :param countries: List of countries
+        :lang lang: Language
+        """
+
+        missing = Counter(countries) if countries is not None else None
+        rest = []
+        dates = dates[::-1]
+        while len(dates) and (len(rest) < n or n == -1):
+          day = dates.pop()
+          flag = True
+          iter = missing.most_common() if missing is not None else [[None, None]]
+          for country, _ in iter:
+            try:
+                download_tokens(day, lang=lang, country=country)
+            except Exception:
+              flag = False
+              if missing is not None:
+                  missing.update([country])  
+              break
+          if flag:
+            rest.append(day)
+        return rest
+
+
 class Tokenize(object):
     """ Tokenize transforms a text into a sequence, where 
     each number identifies a particular token; the q-grams 
@@ -502,4 +529,4 @@ class BagOfWords(SparseMatrix):
 
     def transform(self, data: List[str]) -> csr_matrix:
         """Transform a list of text to a Bag of Words using TFIDF""" 
-        return self.tonp(self._transform(data))
\ No newline at end of file
+        return self.tonp(self._transform(data))
